#!/usr/bin/env python

import sys
import struct
import math
import numpy as np
import matplotlib.pyplot as plt

import colormaps

phi1 = 0.0
phi2 = 2.0*np.pi

GM_SI = 1.32712440018e20
AU_SI = 149597870700.0
MSOL_SI = 1.98855e30
SIGMA2CGS = MSOL_SI*1000.0/(AU_SI*100.0)**2

R_STANDARD = 8.3144598
MMW = 2.4
R_SI = R_STANDARD/(MMW*0.001)
T2SI = GM_SI/R_SI/AU_SI

def readdat(filename, nr, nphi):
    nbytes = nr*nphi*8
    print "filename = " + str(filename)
    print "nbytes = " + str(nbytes)

    f = open(filename, "r")
    data = f.read(nbytes)
    f.close()

    size = len(data)
    fmt = 'd'
    size = size/struct.calcsize(fmt)
    fmt = size*fmt
    data = np.array(struct.unpack(fmt, data)).reshape(nr,nphi)
    return data

def circle(x, y, R, style='w-', linewidth=0.1, color='white'):
    xc = []
    yc = []
    n = 64
    for i in xrange(0,n):
        xc.append(x + R*math.cos(2.0*math.pi*i/(n-1)))
        yc.append(y + R*math.sin(2.0*math.pi*i/(n-1)))

    plt.plot(xc, yc, style, linewidth=linewidth, color=color)

def gastemper(filename, directory="", out=None, interactive=False, nr=1024, nphi=1536, r1=0.2, r2=2.0, c1=None, c2=None, cmap=colormaps.plasma, dpi=300, logscale=False, \
    xc=None, yc=None, xp=None, yp=None, zp=None, Rp=None, RH=None, ip=None):

    if directory != "":
        filename = directory + "/" + filename 
    data = readdat(filename, nr, nphi)
    if logscale:
        r = 10.**(np.linspace(np.log10(r1), np.log10(r2), nr+1))
    else:
        r = np.linspace(r1, r2, nr+1)
    ang = 170./180.*np.pi
    phi = np.linspace(phi1+ang, phi2+ang, nphi+1)
    phi_grid, r_grid = np.meshgrid(phi, r)
    x, y = r_grid*np.cos(phi_grid), r_grid*np.sin(phi_grid)

    mean = np.zeros((nr,nphi))
    mean1 = np.average(data, axis=1)
    for j in xrange(0,nphi):
        mean[:,j] = mean1

    fig = plt.figure(figsize=(3.8,4.0))
#    ax = fig.add_subplot(111)
    ax = fig.add_axes([0.05, 0.125, 0.80, 0.74])
    ax.set_xlabel(r'$r\ [{\rm au}]$', usetex=True, fontsize=12)
    ax.set_aspect('equal')
    ax.set_facecolor('black')
    plt.xticks([0.0, 0.5, 1.0, 1.5])
    plt.yticks([])
    ap = ax.pcolormesh(x, y, (data-mean)*T2SI, cmap=cmap)
    if c1 != None:
        ap.set_clim(c1, c2)

#    if xc == None:
#        ax.set_xlim(-r2, r2)
#        ax.set_ylim(-r2, r2)
#    else:
#        tmp = 0.6
#        ax.set_xlim(xc-tmp, xc+tmp)
#        ax.set_ylim(yc-tmp, yc+tmp)
    r2=1.6
    ax.set_xlim(0.0, r2)
    ax.set_ylim(0.0, r2)

    print "xp = ", xp
    print "yp = ", yp
    print "Rp = ", Rp
    s = ["Mercury", "Venus", "0.5Earth", "0.5Earth", "Mars"]
    if xp != None:
        for i in xrange(0, len(xp)):
            xtmp = xp[i]*np.cos(ang) - yp[i]*np.sin(ang)
            ytmp = xp[i]*np.sin(ang) + yp[i]*np.cos(ang)
            if i == 1:
                xc = xtmp
                yc = ytmp
#            if xc == None or (xp[i] > xc-tmp and xp[i] < xc+tmp and yp[i] > yc-tmp and yp[i] < yc+tmp):
            if xtmp > 0.0 and xtmp < r2 and ytmp > 0.0 and ytmp < r2:
                r_perp = math.sqrt(xp[i]*xp[i]+yp[i]*yp[i])
                r = math.sqrt(xp[i]*xp[i]+yp[i]*yp[i]+zp[i]*zp[i])
                circle(xtmp, ytmp, RH[i], linewidth=0.2, color='white')
                plt.text(xtmp+RH[i], ytmp+RH[i], " %s" % (s[i]), color='white', fontsize=6, weight='light')

#    ax1 = fig.add_axes([0.45, 0.49, 0.35, 0.35]) 
#    plt.xticks([])
#    plt.yticks([])
#    ap1 = ax1.pcolormesh(x, y, (data-mean)*T2SI, cmap=cmap)
#    tmp = 0.075
#    ax1.set_xlim(xc-tmp, xc+tmp)
#    ax1.set_ylim(yc-tmp, yc+tmp)
#    circle(xc, yc, RH[1], linewidth=0.2, color='white')

    ax2 = fig.add_axes([0.0, 0.125, 0.04, 0.74]) 
    cb = plt.colorbar(ap, cax=ax2)
    cb.set_label(r'$T-T(r)\ [{\rm K}]$', usetex=True, fontsize=12)
    ax2.yaxis.set_ticks_position('left')
    ax2.yaxis.set_label_position('left')

    if out == None:
        out = filename + ".png"
    plt.savefig(out, dpi=dpi, bbox_inches='tight', pad_inches=0)
    if interactive:
        plt.show()

def main():
    if len(sys.argv) > 2:
        for filename in sys.argv[1:]:
            gastemper(filename, interactive=False)
    elif len(sys.argv) == 2:
        filename = sys.argv[1]
        gastemper(filename)
    else:
        filename = "gastemper0.dat"
        gastemper(filename)

if __name__ == "__main__":
    main()



