#!/usr/bin/env python

import sys
import struct
import re
import math
import numpy as np
import matplotlib.pyplot as plt

import colormaps

phi1 = 0.0
phi2 = 2.0*np.pi

def readdat(filename, nr, nphi):
    nbytes = nr*nphi*8
    print "filename = " + str(filename)
    print "nbytes = " + str(nbytes)

    f = open(filename, "r")
    data = f.read(nbytes)
    f.close()

    size = len(data)
    fmt = 'd'
    size = size/struct.calcsize(fmt)
    fmt = size*fmt
    data = np.array(struct.unpack(fmt, data)).reshape(nr,nphi)
    return data

def circle(x, y, R):
    xc = []
    yc = []
    n = 64
    for i in xrange(0,n):
        xc.append(x + R*math.cos(2.0*math.pi*i/(n-1)))
        yc.append(y + R*math.sin(2.0*math.pi*i/(n-1)))

    plt.plot(xc, yc, 'k-', linewidth=0.1)

def gasdens(filename, out=None, interactive=False, nr=1024, nphi=1536, r1=0.2, r2=2.0, \
    logscale=False, cmap=colormaps.magma, c1=None, c2=None, \
    xc=None, yc=None, xp=None, yp=None, zp=None, Rp=None):

    data = readdat(filename, nr, nphi)
    print "logscale = ", logscale
    if logscale:
        r = 10.**(np.linspace(np.log10(r1), np.log10(r2), nr+1))
    else:
        r = np.linspace(r1, r2, nr+1)
    phi = np.linspace(phi1, phi2, nphi+1)
    phi_grid, r_grid = np.meshgrid(phi, r)
    x, y = r_grid*np.cos(phi_grid), r_grid*np.sin(phi_grid)

    mean = np.zeros((nr,nphi))
    mean1 = np.average(data, axis=1)
    for j in xrange(0,nphi):
        mean[:,j] = mean1

    fig = plt.figure(figsize=(6.0,4.0))
    ax = fig.add_subplot(111)
    ax.set_xlabel(r'$x\ [{\rm au}]$', usetex=True, fontsize=12)
    ax.set_ylabel(r'$y\ [{\rm au}]$', usetex=True, fontsize=12)
    ax.set_aspect('equal')
    ax.set_xlim(-r2, r2)
    ax.set_ylim(-r2, r2)
    plt.yticks([-2.0, -1.0, 0.0, 1.0, 2.0])
    ax.set_facecolor('black')
    ap = ax.pcolormesh(x, y, (data-mean)/mean, cmap=cmap)
    if c1 != None:
      ap.set_clim(c1,c2)
    cb = plt.colorbar(ap)
    cb.set_label(r'$(\Sigma-\Sigma(r))/\Sigma(r)$', usetex=True, fontsize=12)

    if xc != None:
        print "xc = ", xc, " au"
        print "yc = ", yc, " au"
        tmp = 2.0
        ax.set_xlim(xc-tmp, xc+tmp)
        ax.set_ylim(yc-tmp, yc+tmp)
#        ax.set_xlim(0, xc+tmp)
#        ax.set_ylim(0, yc+tmp)

    if xp != None:
        print "xp = ", xp, " au"
        print "yp = ", yp, " au"
#        plt.plot(xp, yp, 'w+', markerfacecolor='none', markeredgecolor='white', markeredgewidth=0.1)
#        s = [ "Mercury", "Venus", "0.5 Earth", "0.5 Earth", "Mars" ]
        for i in xrange(0, len(xp)):
            r_perp = math.sqrt(xp[i]*xp[i]+yp[i]*yp[i])
            r = math.sqrt(xp[i]*xp[i]+yp[i]*yp[i]+zp[i]*zp[i])
            circle(xp[i], yp[i], Rp[i])
#            plt.text(xp[i]+8.0*Rp[i], yp[i]+Rp[i], "%s" % (s[i]), color='black', fontsize=6, weight='light')

    if out == None:
        out = filename + ".png"
    plt.savefig(out, dpi=600, bbox_inches='tight', pad_inches=0)
    if interactive:
        plt.show()
    plt.close()

def interp_lin(x1, x2, y1, y2, x):
    """Linear interpolation"""
    return y1 + (y2-y1)*(x-x1)/(x2-x1)

def interp_near(x1, x2, y1, y2, x):
    """Nearest-neigbour interpolation"""
    if abs(x1-x) < abs(x2-x):
        return y1
    else:
        return y2

def gasdens_nbody(filename, filenbody, out=None, cmap=colormaps.plasma, nr=1024, nphi=1536, Ninterm=500, logscale=False, c1=None, c2=None):

    t = []
    x = []
    y = []
    z = []
    m = []
    ttmp = []
    xtmp = []
    ytmp = []
    ztmp = []
    mtmp = []

    f = open(filenbody, "r")
    for line in f.readlines():
        l = line.split()
        i = int(l[0])

        if i == 1 and len(xtmp) > 0:
            t.append(ttmp[0])
            x.append(xtmp)
            y.append(ytmp)
            z.append(ztmp)
            m.append(mtmp)
            ttmp = []
            xtmp = []
            ytmp = []
            ztmp = []
            mtmp = []

        ttmp.append(float(l[1]))
        xtmp.append(float(l[12]))   
        ytmp.append(float(l[13]))
        ztmp.append(float(l[14]))
        mtmp.append(float(l[8]))
    
    f.close()

    j = 1
    dt = 0.314159
    i = int(re.sub(r".*gasdens([0-9]+).dat", r"\1", filename))
    print "i = ", i
    tmp = i*dt*Ninterm
    print "tmp = ", tmp
    while t[j] < tmp and j < len(t)-1:
        j += 1
    print "j = ", j
    print "t[j-1] = ", t[j-1]
    print "t[j] = ", t[j]

    xp = []
    yp = []
    zp = []
    mp = []
    Rp = []
    j0 = j
    for j in xrange(j0,j0+1):
        for k in xrange(0,len(x[j])):
            xp.append(interp_near(t[j-1], t[j], x[j-1][k], x[j][k], tmp))
            yp.append(interp_near(t[j-1], t[j], y[j-1][k], y[j][k], tmp))
            zp.append(interp_near(t[j-1], t[j], z[j-1][k], z[j][k], tmp))
            mp.append(interp_near(t[j-1], t[j], m[j-1][k], m[j][k], tmp))
 
            r = math.sqrt(xp[k]*xp[k] + yp[k]*yp[k] + zp[k]*zp[k])
            print "r = ", r, " au"
            R_Hill = r*pow(mp[k]/(3.0*1.0), 1./3.)
            Rp.append(R_Hill)

    gasdens(filename, out, xc=0.0, yc=0.0, xp=xp, yp=yp, zp=zp, Rp=Rp, nr=nr, nphi=nphi, logscale=logscale, cmap=cmap, c1=c1, c2=c2)

def main():
    if len(sys.argv) > 2:
        for filename in sys.argv[1:]:
            gasdens(filename, interactive=False)
    elif len(sys.argv) == 2:
        filename = sys.argv[1]
        gasdens(filename)
    else:
        filename = "gasdens0.dat"
        gasdens(filename)

if __name__ == "__main__":
    main()


