#!/usr/bin/env python

import sys
import struct
import math
import numpy as np
import matplotlib.pyplot as plt

import colormaps

phi1 = 0.0
phi2 = 2.0*np.pi

GM_SI = 1.32712440018e20
AU_SI = 149597870700.0
MSOL_SI = 1.98855e30
SIGMA2CGS = MSOL_SI*1000.0/(AU_SI*100.0)**2

R_STANDARD = 8.3144598
MMW = 2.4
R_SI = R_STANDARD/(MMW*0.001)
T2SI = GM_SI/R_SI/AU_SI

ADIABIND = 1.4
MOLWEIGTH = 1.0
GASCONST = 1.0
SQRT_ADIABIND_INV = 1.0/math.sqrt(ADIABIND)

def Omega(r):
    return r**(-1.5)
def cs(T):
    return math.sqrt(ADIABIND*T/MOLWEIGTH*GASCONST)

def H_func(r,T):
    return cs(T)/Omega(r)*SQRT_ADIABIND_INV

def readdat(filename, nr, nphi):
    nbytes = nr*nphi*8
    print "filename = " + str(filename)
    print "nbytes = " + str(nbytes)

    f = open(filename, "r")
    data = f.read(nbytes)
    f.close()

    size = len(data)
    fmt = 'd'
    size = size/struct.calcsize(fmt)
    fmt = size*fmt
    data = np.array(struct.unpack(fmt, data)).reshape(nr,nphi)
    return data

def circle(x, y, R, linewidth=0.1):
    xc = []
    yc = []
    n = 64
    for i in xrange(0,n):
        xc.append(x + R*math.cos(2.0*math.pi*i/(n-1)))
        yc.append(y + R*math.sin(2.0*math.pi*i/(n-1)))

    plt.plot(xc, yc, 'w-', linewidth=linewidth)

def rho(filename, out=None, interactive=False, nr=1024, nphi=1536, r1=0.2, r2=2.0, \
    c1=None, c2=None, cmap=colormaps.plasma, dpi=300, logscale=False, shrink=1.0, ticks=None, \
    xc=None, yc=None, xp=None, yp=None, zp=None, Rp=None, ip=None):

    if logscale:
        r = 10.**(np.linspace(np.log10(r1), np.log10(r2), nr+1))
    else:
        r = np.linspace(r1, r2, nr+1)
#    phi = np.linspace(phi1, phi2, nphi+1)
#    phi_grid, r_grid = np.meshgrid(phi, r)
#    x, y = r_grid*np.cos(phi_grid), r_grid*np.sin(phi_grid)

    T = readdat(filename, nr, nphi)
    Sigma = readdat(filename.replace("temper","dens"), nr, nphi)

    ntheta = 240
    theta1 = -0.3
    theta2 = -theta1
    theta = np.linspace(theta1, theta2, ntheta+1)
    theta_grid, r_grid = np.meshgrid(theta, r)
    x, z = r_grid, r_grid*np.sin(theta_grid)

    H = np.zeros((nr+1))
    rho = np.zeros((nr+1, ntheta+1))
    j = 0
    for i in xrange(0,nr):
        for k in xrange(0,ntheta):
            H[i] = H_func(r[i], T[i,j])
            rho[i,k] = Sigma[i,j]/(np.sqrt(2.0*np.pi)*H[i]) * np.exp(-(z[i,k]/(2.0*H[i]))**2)

    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.set_xlabel(r'$r_\perp\ [{\rm au}]$', usetex=True)
    ax.set_ylabel(r'$z\ [{\rm au}]$', usetex=True)
    ax.set_xlim(r1, r2)
    tmp = 0.5
    ax.set_ylim(-tmp, tmp)
    ax.set_aspect('equal')
    ax.set_facecolor('white')
    ap = ax.pcolormesh(x, z, np.where(rho > 1.e-16, np.log10(rho), -16.0), cmap=cmap)
    if c1 != None:
        ap.set_clim(c1, c2)
#    cb = plt.colorbar(ap, fraction=0.0315, pad=0.01)
    cb = plt.colorbar(ap, shrink=shrink, pad=0.01, ticks=ticks)
    cb.set_label(r'$\log\rho\ [M_{\!\odot}/{\rm au}^3]$', usetex=True)

    ax.plot(x, H, ',', color='lime')
    ax.plot(x, -H, ',', color='lime')
    ax.plot(x, 3*H, ',', color='lime')
    ax.plot(x, -3*H, ',', color='lime')
    ax.plot([r1,r2], [0,0], ':w', linewidth=0.3)

#    if xc != None:
#        print "xc = ", xc, " au"
#        print "yc = ", yc, " au"
#        tmp = 1.0
#        ax.set_xlim(xc-tmp, xc+tmp)
#        ax.set_ylim(yc-tmp, yc+tmp)

    print "xp = ", xp
    print "zp = ", zp
    print "Rp = ", Rp
    if xp != None:
        for i in xrange(0, len(xp)):
            r_perp = math.sqrt(xp[i]*xp[i]+yp[i]*yp[i])
            r = math.sqrt(xp[i]*xp[i]+yp[i]*yp[i]+zp[i]*zp[i])
            z_ = r*math.sin(ip[i])
            circle(r_perp, z_, Rp[i], linewidth=0.2)
            if Rp[i] < 0.01:
                plt.plot(r_perp, z_, 'w+', markerfacecolor='none', markeredgecolor='white', markersize=2.0, markeredgewidth=0.1)
#            plt.text(xp[i]+Rp[i], zp[i]+Rp[i], "r = %.2f\nR_H = %.4f" % (r, Rp[i]), color='white', fontsize=4, weight='light')

    if out == None:
        out = filename + ".png"
    plt.savefig(out, dpi=dpi, bbox_inches='tight', pad_inches=0)
    if interactive:
        plt.show()

def main():
    if len(sys.argv) > 2:
        for filename in sys.argv[1:]:
            rho(filename, interactive=False)
    elif len(sys.argv) == 2:
        filename = sys.argv[1]
        rho(filename)
    else:
        filename = "rho0.dat"
        rho(filename)

if __name__ == "__main__":
    main()



